__title__ = 'flip-data'
__package_name__ = 'flip'
__version__ = '0.1.9'
__description__ = 'Generate thousands of new 2D images from a small batch of objects and backgrounds.'
__email__ = 'd@linkedai.co'
__author__ = 'Diego Parra'
__github__ = 'https://github.com/LinkedAi/flip'
__pypi__ = 'https://pypi.org/project/flip-data/'
__license__ = 'GPL-3.0'
